@@ -1,4 +1,4 @@
-/* SPDX-License-Identifier: GPL-2.0
+/* SPDX-License-Identifier: MIT
  *
  * Copyright (C) 2015-2018 Jason A. Donenfeld <Jason@zx2c4.com>. All Rights Reserved.
  *
@@ -12,23 +12,30 @@
 #include <asm/unaligned.h>
 #include <linux/kernel.h>
 #include <linux/string.h>
+#include <linux/module.h>
+#include <linux/init.h>
 
-#ifndef HAVE_POLY1305_ARCH_IMPLEMENTATION
+#if defined(CONFIG_ZINC_ARCH_X86_64)
+#include "poly1305-x86_64-glue.h"
+#elif defined(CONFIG_ZINC_ARCH_ARM) || defined(CONFIG_ZINC_ARCH_ARM64)
+#include "poly1305-arm-glue.h"
+#elif defined(CONFIG_ZINC_ARCH_MIPS) || defined(CONFIG_ZINC_ARCH_MIPS64)
+#include "poly1305-mips-glue.h"
+#else
 static inline bool poly1305_init_arch(void *ctx,
-				      const u8 key[POLY1305_KEY_SIZE],
-				      simd_context_t simd_context)
+				      const u8 key[POLY1305_KEY_SIZE])
 {
 	return false;
 }
 static inline bool poly1305_blocks_arch(void *ctx, const u8 *input,
 					const size_t len, const u32 padbit,
-					simd_context_t simd_context)
+					simd_context_t *simd_context)
 {
 	return false;
 }
 static inline bool poly1305_emit_arch(void *ctx, u8 mac[POLY1305_MAC_SIZE],
 				      const u32 nonce[4],
-				      simd_context_t simd_context)
+				      simd_context_t *simd_context)
 {
 	return false;
 }
@@ -43,15 +50,14 @@ void __init poly1305_fpu_init(void)
 #include "poly1305-donna32.h"
 #endif
 
-void poly1305_init(struct poly1305_ctx *ctx, const u8 key[POLY1305_KEY_SIZE],
-		   simd_context_t simd_context)
+void poly1305_init(struct poly1305_ctx *ctx, const u8 key[POLY1305_KEY_SIZE])
 {
 	ctx->nonce[0] = get_unaligned_le32(&key[16]);
 	ctx->nonce[1] = get_unaligned_le32(&key[20]);
 	ctx->nonce[2] = get_unaligned_le32(&key[24]);
 	ctx->nonce[3] = get_unaligned_le32(&key[28]);
 
-	if (!poly1305_init_arch(ctx->opaque, key, simd_context))
+	if (!poly1305_init_arch(ctx->opaque, key))
 		poly1305_init_generic(ctx->opaque, key);
 
 	ctx->num = 0;
@@ -60,7 +66,7 @@ EXPORT_SYMBOL(poly1305_init);
 
 static inline void poly1305_blocks(void *ctx, const u8 *input, const size_t len,
 				   const u32 padbit,
-				   simd_context_t simd_context)
+				   simd_context_t *simd_context)
 {
 	if (!poly1305_blocks_arch(ctx, input, len, padbit, simd_context))
 		poly1305_blocks_generic(ctx, input, len, padbit);
@@ -68,14 +74,14 @@ static inline void poly1305_blocks(void *ctx, const u8 *input, const size_t len,
 
 static inline void poly1305_emit(void *ctx, u8 mac[POLY1305_KEY_SIZE],
 				 const u32 nonce[4],
-				 simd_context_t simd_context)
+				 simd_context_t *simd_context)
 {
 	if (!poly1305_emit_arch(ctx, mac, nonce, simd_context))
 		poly1305_emit_generic(ctx, mac, nonce);
 }
 
 void poly1305_update(struct poly1305_ctx *ctx, const u8 *input, size_t len,
-		     simd_context_t simd_context)
+		     simd_context_t *simd_context)
 {
 	const size_t num = ctx->num % POLY1305_BLOCK_SIZE;
 	size_t rem;
@@ -109,8 +115,8 @@ void poly1305_update(struct poly1305_ctx *ctx, const u8 *input, size_t len,
 }
 EXPORT_SYMBOL(poly1305_update);
 
-void poly1305_finish(struct poly1305_ctx *ctx, u8 mac[POLY1305_MAC_SIZE],
-		     simd_context_t simd_context)
+void poly1305_final(struct poly1305_ctx *ctx, u8 mac[POLY1305_MAC_SIZE],
+		    simd_context_t *simd_context)
 {
 	size_t num = ctx->num % POLY1305_BLOCK_SIZE;
 
@@ -126,6 +132,36 @@ void poly1305_finish(struct poly1305_ctx *ctx, u8 mac[POLY1305_MAC_SIZE],
 
 	memzero_explicit(ctx, sizeof(*ctx));
 }
-EXPORT_SYMBOL(poly1305_finish);
+EXPORT_SYMBOL(poly1305_final);
 
 #include "../selftest/poly1305.h"
+
+static bool nosimd __initdata = false;
+
+#ifndef COMPAT_ZINC_IS_A_MODULE
+int __init poly1305_mod_init(void)
+#else
+static int __init mod_init(void)
+#endif
+{
+	if (!nosimd)
+		poly1305_fpu_init();
+#ifdef DEBUG
+	if (!poly1305_selftest())
+		return -ENOTRECOVERABLE;
+#endif
+	return 0;
+}
+
+#ifdef COMPAT_ZINC_IS_A_MODULE
+static void __exit mod_exit(void)
+{
+}
+
+module_param(nosimd, bool, 0);
+module_init(mod_init);
+module_exit(mod_exit);
+MODULE_LICENSE("GPL v2");
+MODULE_DESCRIPTION("Poly1305 one-time MAC");
+MODULE_AUTHOR("Jason A. Donenfeld <Jason@zx2c4.com>");
+#endif
