@@ -17,9 +17,8 @@
 #include <net/udp_tunnel.h>
 #include <net/ipv6.h>
 
-static inline int send4(struct wireguard_device *wg, struct sk_buff *skb,
-			struct endpoint *endpoint, u8 ds,
-			struct dst_cache *cache)
+static int send4(struct wireguard_device *wg, struct sk_buff *skb,
+		 struct endpoint *endpoint, u8 ds, struct dst_cache *cache)
 {
 	struct flowi4 fl = {
 		.saddr = endpoint->src4.s_addr,
@@ -99,9 +98,8 @@ out:
 	return ret;
 }
 
-static inline int send6(struct wireguard_device *wg, struct sk_buff *skb,
-			struct endpoint *endpoint, u8 ds,
-			struct dst_cache *cache)
+static int send6(struct wireguard_device *wg, struct sk_buff *skb,
+		 struct endpoint *endpoint, u8 ds, struct dst_cache *cache)
 {
 #if IS_ENABLED(CONFIG_IPV6)
 	struct flowi6 fl = {
@@ -264,8 +262,7 @@ int socket_endpoint_from_skb(struct endpoint *endpoint,
 	return 0;
 }
 
-static inline bool endpoint_eq(const struct endpoint *a,
-			       const struct endpoint *b)
+static bool endpoint_eq(const struct endpoint *a, const struct endpoint *b)
 {
 	return (a->addr.sa_family == AF_INET && b->addr.sa_family == AF_INET &&
 		a->addr4.sin_port == b->addr4.sin_port &&
@@ -339,7 +336,7 @@ err:
 	return 0;
 }
 
-static inline void sock_free(struct sock *sock)
+static void sock_free(struct sock *sock)
 {
 	if (unlikely(!sock))
 		return;
@@ -347,7 +344,7 @@ static inline void sock_free(struct sock *sock)
 	udp_tunnel_sock_release(sock->sk_socket);
 }
 
-static inline void set_sock_opts(struct socket *sock)
+static void set_sock_opts(struct socket *sock)
 {
 	sock->sk->sk_allocation = GFP_ATOMIC;
 	sock->sk->sk_sndbuf = INT_MAX;
